#ifndef __PROTOCOL_H__
#define __PROTOCOL_H__

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <string.h>
#include <sys/shm.h>  // shared memory
#include <sys/sem.h>  // semaphore
#include <sys/msg.h>  // message queue


#ifdef DEBUG
    #define dbg_print(src,args...) printf(src,##args)
#else
    #define dbg_print(src,args...)
#endif

#define DEVICE_ADDR 0xFFFFFFFF

#define CMD_CANCEL          0x30
#define CMD_AUTO_ENROLL     0X31
#define CMD_AUTO_IDENTIFY   0X32
#define CMD_GET_CHIPSN      0x34
#define CMD_SET_EMPTY       0x0d
#define CMD_GET_SYSPAR      0x0f
#define CMD_GET_ValTempNum  0x1d    /* ValidTempleteNum */
#define CMD_GET_INDEXTABLE  0x1f
#define CMD_DEL_TEMPLETE    0x0c
#define CMD_GET_IMAGE       0X3D

#define FP_SPEED    57600
#define FP_DATABITS 8
#define FP_STOPBITS 2
#define FP_PARITY   'n'

#define FINGERPRINT_DEV "/dev/ttyS1"

#define SEM_KEY 12580

struct recvPack
{
    unsigned char pack_ID;
    unsigned short data_len;
    unsigned char ack_code;
    unsigned char buff[128];
};

/*数据收发*/
void sendCMD(int fd, unsigned char cmd_code, unsigned char p_argc, unsigned char *p_arg);
int recvPackage(int fd, struct recvPack *rp);
void printResp(struct recvPack *rp);

/*fingerprint operate*/
int fp_open(void);
int fp_close(int fd);
unsigned short fp_get_valid_templete_num(int fd);
unsigned char pre_fp_enroll(int fd, unsigned short id, unsigned char times);
int circ_enroll(int fd);
unsigned short fp_verify(int fd);
int fp_delete(int fd, int id, int num);
int fp_empty(int fd);
int PS_rec(int fd);
unsigned char PS_Cancel(int fd);

#endif
